
/* 
 * @param {Object} args
 */

var ImageProcessing = require('/ui/common/ImageProcessing');

var _PlatWidth = Titanium.Platform.displayCaps.platformWidth;
var _PlatHeight = Titanium.Platform.displayCaps.platformHeight;

exports.imageCapture = function(args) {

    var ImageCaptureUtils = require('/ui/common/IphoneImageCaptureUtils').getInstance();
    /*
     * UI Elements
     */
    var overlay = Ti.UI.createView({
        width: _PlatWidth,
        height: _PlatHeight,
    });

    _PlatWidth = Titanium.Platform.displayCaps.platformWidth;
    _PlatHeight = Titanium.Platform.displayCaps.platformHeight;
    Ti.API.error("_PlatHeight: " + _PlatHeight + " _PlatWidth: " + _PlatWidth);
    
    var activityIndicator = new ActivityIndicator();
    overlay.add(activityIndicator.getTiView());

    activityIndicator.start();

    var capturedImageView = null;
    
    var container = Ti.UI.createView({
        transform: Ti.UI.create2DMatrix({rotate: 90}),
        width: _PlatHeight,
        height: _PlatWidth,
        layout: 'horizontal',
        zIndex: 2
    });
    overlay.add(container);
    
    var leftColumnWidth = container.width - 60;
    var leftColumn = Ti.UI.createView({
        width: leftColumnWidth,
        height: '100%'
    });
    container.add(leftColumn);
    
    var rightColumn = Ti.UI.createView({
        width: 60,
        opacity: 0.6,
        height: '100%',
        backgroundGradient: {
            colors: ['#303030', '#202020', '#181818'],
            startPoint: {'x': 0.0, 'y': 0.0},
            endPoint: {'x': 60, 'y': 0}
        }
    });
    container.add(rightColumn);
    
    var cameraAlignImage = Ti.UI.createImageView({
        image: 'images/camera-align.png',
        left: 10,
        right: 10,
        zIndex: 1
    });

    //For the iPhone's with a taller display we need to use a different camera align image.
    if(ImageCaptureUtils.isTallDisplay()){
        cameraAlignImage.image = 'images/camera-align-ip5.png'; 
        cameraAlignImage.right = 16;
        cameraAlignImage.left = 16;
    }

    var captureViewLabel = Ti.UI.createLabel({
        top: 20,
        color: 'red',
        font: {fontSize: 40, font: 'Helvetica Neue'},
        text: "Capture View"
    });

    cameraAlignImage.add(captureViewLabel);

    leftColumn.add(cameraAlignImage);
    
    var helpView = Ti.UI.createView({
        top: 0,
        left: 0,
        right: 0,
        width: leftColumnWidth,
        height: 96,
        backgroundColor: 'black',
        opacity: 0.75,
        visible: false,
        zIndex: 2
    });
    leftColumn.add(helpView);
    
    var helpLabel = Ti.UI.createLabel({
        top: 10,
        left: 26,
        right: 16,
        text: args.overlayText,
        color: 'white',
        font: {fontSize: 14, font: 'Helvetica Neue'}
    });
    helpView.add(helpLabel);
    
    var helpButton = Ti.UI.createButton({
        width: 50,
        height: 50,
        backgroundImage: 'images/info.png',
        top: (rightColumn.width-50)/2
    });
    rightColumn.add(helpButton);
    
    var cameraButton = Ti.UI.createButton({
        top: _PlatWidth/2 - 25,
        left: 5,
        backgroundImage: 'images/camera_up.png',
        backgroundSelectedImage: 'images/camera_down.png',
        backgroundDisabledImage: 'images/camera_down.png',
        width: 49,
        height: 49
    });
    rightColumn.add(cameraButton);
    
    var cancelButton = Ti.UI.createButton({
        width: 50,
        height: 50,
        backgroundImage: 'images/rdc-close.png',
        backgroundDisabledImage: 'images/rdc-close.png',
        bottom: (rightColumn.width-50)/2
    });
    rightColumn.add(cancelButton);
    
    var acceptButton = Ti.UI.createButton({
        backgroundImage: 'images/accept_btn_up.png',
        backgroundSelectedImage: 'images/accept_btn_down.png',
        backgroundDisabledImage: 'images/accept_btn_down.png',
        width: 49,
        height: 49,
        visible: false
    });
    rightColumn.add(acceptButton);
    
    var retakeButton = Ti.UI.createButton({
        width: 50,
        height: 50,
        backgroundImage: 'images/retake_btn.png',
        backgroundDisabledImage: 'images/retake_btn.png',
        bottom: (rightColumn.width-50)/2,
        visible: false
    });
    rightColumn.add(retakeButton);

    /*
     * Button Handlers
     */
    
    // Camera button handler captures an image and displays it in the overlay view, above
    // the camera preview, and replaces the camera and cancel buttons with accept
    // and retake buttons.
    function captureButtonHandler() {
        cameraButton.setTouchEnabled(false);
        cancelButton.setTouchEnabled(false);
        Ti.Media.takePicture();
    }

    // Cancel button handler hides the camera and cancels image capture
    function cancelButtonHandler() {
        Ti.Media.hideCamera();
        args.cancel();
        endImageCapture();
    }

     // Accept button handler hides the camera preview and starts image processing
    function acceptButtonHandler() {
        acceptButton.setTouchEnabled(false);
        retakeButton.setTouchEnabled(false);
        // invoke callback to signal that processing has started
        args.processing();
        Ti.Media.hideCamera();

        var original = capturedImage;
        // Rotate the Image
        capturedImage = ImageProcessing.rotateImage(capturedImage);
        // Crop the Image
        capturedImage = ImageCaptureUtils.cropImage(capturedImage, ImageCaptureUtils.getCropProperties());

        var helperView = Ti.UI.createView(); 

        ImageProcessing.processCheckImage(capturedImage, original, 1600, 0.7, true, args.success, args.error, helpView);
            

        endImageCapture();
    }

    // Retake button handler removes the captured image from view, resuming the
    // camera preview
    function retakeButtonHandler() {
        hideCapturedImage();
        //Add used because using setVisible allowed the camraAlignImage to shift to the upper right corner
        leftColumn.add(cameraAlignImage);
        cameraButton.setVisible(true);
        cameraButton.setTouchEnabled(true);
        cancelButton.setTouchEnabled(true);
        acceptButton.setVisible(false);
        cancelButton.setVisible(true);
        retakeButton.setVisible(false);
    }

    // Help button handler shows/hides the help overlay
    function helpButtonHandler() {
        helpView.setVisible(!helpView.getVisible());
    }

    // /*
    //  * Event Listeners
    //  */
    function setupEventListeners() {
        cameraButton.addEventListener('click', captureButtonHandler);
        cancelButton.addEventListener('click', cancelButtonHandler);
        acceptButton.addEventListener('click', acceptButtonHandler);
        retakeButton.addEventListener('click', retakeButtonHandler);
        helpButton.addEventListener('click', helpButtonHandler);
        //Stop and hide the indicator now that the buttons can be pressed. 
        activityIndicator.stop();
    }

    function endImageCapture() {
        capturedImage = null;
        // Remove the event listeners added
        cameraButton.removeEventListener('click', captureButtonHandler);
        cancelButton.removeEventListener('click', cancelButtonHandler);
        acceptButton.removeEventListener('click', acceptButtonHandler);
        retakeButton.removeEventListener('click', retakeButtonHandler);
        helpButton.removeEventListener('click', helpButtonHandler);
    }

    /* Helper methods to show/hide the captured check image */
    function showCapturedImage() {
        capturedImageView = Ti.UI.createImageView({
            autorotate: false, //Prevents the image from autorotating based on the exif image data
            image: capturedImage,
            height: _PlatWidth,
            right: ImageCaptureUtils.getPreviewRight(), 
            width: Ti.UI.SIZE,
            zIndex: 1,
            opacity: 0.6
        });

        var previewViewLabel = Ti.UI.createLabel({
            top: 100,
            color: 'blue',
            font: {fontSize: 40, font: 'Helvetica Neue'},
            text: "Preview View"
        });

        capturedImageView.add(previewViewLabel);

        // If we are using the Default properties then we need to show the user
        // the entire image even if its stretched since we do not know what was 
        // shown to the user during preview. This should not happen in the real world
        // but just in case this fallback exists so that they user can still capture their 
        // images.
        if(ImageCaptureUtils.getDescription() == "Default"){
            capturedImageView.width = _PlatHeight - 60;
        }

        //Use transform to scale the image up so that it matches the one shown to the user during image capture.
        var previewRotateTransform = Ti.UI.create2DMatrix({
            scale: ImageCaptureUtils.getPreviewScale()
        });   

        capturedImageView.setTransform(previewRotateTransform);

        leftColumn.add(capturedImageView);
    }
    
    function hideCapturedImage() {
        leftColumn.remove(capturedImageView);
        capturedImageView = null;
    }

    // Create Transform for the viewFinder so that we can scale it appropriately depending on the device.
    var cameraTransform = Ti.UI.create2DMatrix({
        scale: ImageCaptureUtils.getViewFinderScale()
    });
    
    // Show the camera
    Titanium.Media.showCamera({
        success : function(event) {
            activityIndicator.startDarkIndicator();
            capturedImage = event.media;
            event.media = null;
            event = null;
            Ti.API.error('Image Captured, Dims: ' +  capturedImage.width + ',' + capturedImage.height + " Aspect Ratio: " + capturedImage.width/capturedImage.height);

            showCapturedImage();
            //Remove used because using setVisible allowed the camraAlignImage to shift to the upper right corner
            leftColumn.remove(cameraAlignImage);
            helpView.setVisible(false);
            cameraButton.setVisible(false);
            acceptButton.setVisible(true);
            cancelButton.setVisible(false);
            retakeButton.setVisible(true);
            activityIndicator.stop();
        },
        cancel : function() {
            args.cancel();
        },
        error : function(error) {
            args.error(error);
        },
        saveToPhotoGallery : false,
        allowEditing : false,
        mediaTypes : [Ti.Media.MEDIA_TYPE_PHOTO],
        overlay : overlay,
        showControls : false,
        autohide: false,
        transform:cameraTransform
    });
    // // Setup the event listeners after we call showCamera so that the touch event listeners work. 
    setupEventListeners();
};
    
/**
 * Activity indicator to show while the check image is being processed.
 */
var ActivityIndicator = function() {

    var background = Titanium.UI.createView({
        backgroundColor:'black',
        opacity: 0,
        top: 0,
        left: 0,
        width: '100%',
        height:_PlatHeight,
        zIndex: 1000
    });
    
    var indicatorPlacementFromTop = (_PlatHeight/2)-60-20/2;
    var darkIndicatorHeight = _PlatHeight-60;

    var activityIndicator = Titanium.UI.createActivityIndicator({
        top: indicatorPlacementFromTop,
        height:100,
        width:100,
        color:'white',
        font:{fontWeight:'bold'},
        message: '',
    });

    background.add(activityIndicator);

    var inAnimation = Ti.UI.createAnimation({
        opacity: 0.7,
        duration:500
    });

    //Dark indicator for use right after a picture is taken
    var inDarkAnimation = Ti.UI.createAnimation({
        opacity: 0.9,
        duration:500
    });

    var outAnimation = Ti.UI.createAnimation({
        opacity: 0,
        duration:500
    });

    this.getTiView = function() {
        return background;
    };
    
    this.start = function() {
        activityIndicator.show();
        background.animate(inAnimation);
    };

    this.startDarkIndicator = function() {
        activityIndicator.show();
        background.setHeight(darkIndicatorHeight);
        background.animate(inDarkAnimation);
    };
    
    this.stop = function() {
        background.animate(outAnimation, function() {
            activityIndicator.hide();
        });
    };
};
